from typing import Tuple
import pandas as pd

from aeromaps.models.base import AeromapsModel


class NOxEmissionIndex(AeromapsModel):
    def __init__(self, name="nox_emission_index", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        emission_index_nox_biofuel_2019: float = 0.0,
        emission_index_nox_electrofuel_2019: float = 0.0,
        emission_index_nox_kerosene_2019: float = 0.0,
        emission_index_nox_hydrogen_2019: float = 0.0,
        emission_index_nox_dropin_fuel_evolution: float = 0.0,
        emission_index_nox_hydrogen_evolution: float = 0.0,
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series]:
        """NOX emission index calculation."""

        # Initialization
        for k in range(self.historic_start_year, self.prospection_start_year):
            self.df.loc[k, "emission_index_nox_biofuel"] = emission_index_nox_biofuel_2019
            self.df.loc[k, "emission_index_nox_electrofuel"] = emission_index_nox_electrofuel_2019
            self.df.loc[k, "emission_index_nox_kerosene"] = emission_index_nox_kerosene_2019
            self.df.loc[k, "emission_index_nox_hydrogen"] = emission_index_nox_hydrogen_2019

        # Calculation
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "emission_index_nox_biofuel"] = self.df.loc[
                k - 1, "emission_index_nox_biofuel"
            ] * (1 + emission_index_nox_dropin_fuel_evolution / 100)
            self.df.loc[k, "emission_index_nox_electrofuel"] = self.df.loc[
                k - 1, "emission_index_nox_electrofuel"
            ] * (1 + emission_index_nox_dropin_fuel_evolution / 100)
            self.df.loc[k, "emission_index_nox_kerosene"] = self.df.loc[
                k - 1, "emission_index_nox_kerosene"
            ] * (1 + emission_index_nox_dropin_fuel_evolution / 100)
            self.df.loc[k, "emission_index_nox_hydrogen"] = self.df.loc[
                k - 1, "emission_index_nox_hydrogen"
            ] * (1 + emission_index_nox_hydrogen_evolution / 100)

        emission_index_nox_biofuel = self.df["emission_index_nox_biofuel"]
        emission_index_nox_electrofuel = self.df["emission_index_nox_electrofuel"]
        emission_index_nox_kerosene = self.df["emission_index_nox_kerosene"]
        emission_index_nox_hydrogen = self.df["emission_index_nox_hydrogen"]

        return (
            emission_index_nox_biofuel,
            emission_index_nox_electrofuel,
            emission_index_nox_kerosene,
            emission_index_nox_hydrogen,
        )


class SootEmissionIndex(AeromapsModel):
    def __init__(self, name="nox_emission_index", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        emission_index_soot_biofuel_2019: float = 0.0,
        emission_index_soot_electrofuel_2019: float = 0.0,
        emission_index_soot_kerosene_2019: float = 0.0,
        emission_index_soot_hydrogen_2019: float = 0.0,
        emission_index_soot_dropin_fuel_evolution: float = 0.0,
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series]:
        """Soot emission index calculation."""

        # Initialization
        for k in range(self.historic_start_year, self.prospection_start_year):
            self.df.loc[k, "emission_index_soot_biofuel"] = emission_index_soot_biofuel_2019
            self.df.loc[k, "emission_index_soot_electrofuel"] = emission_index_soot_electrofuel_2019
            self.df.loc[k, "emission_index_soot_kerosene"] = emission_index_soot_kerosene_2019
            self.df.loc[k, "emission_index_soot_hydrogen"] = emission_index_soot_hydrogen_2019

        # Calculation
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "emission_index_soot_biofuel"] = self.df.loc[
                k - 1, "emission_index_soot_biofuel"
            ] * (1 + emission_index_soot_dropin_fuel_evolution / 100)
            self.df.loc[k, "emission_index_soot_electrofuel"] = self.df.loc[
                k - 1, "emission_index_soot_electrofuel"
            ] * (1 + emission_index_soot_dropin_fuel_evolution / 100)
            self.df.loc[k, "emission_index_soot_kerosene"] = self.df.loc[
                k - 1, "emission_index_soot_kerosene"
            ] * (1 + emission_index_soot_dropin_fuel_evolution / 100)
            self.df.loc[k, "emission_index_soot_hydrogen"] = emission_index_soot_hydrogen_2019

        emission_index_soot_biofuel = self.df["emission_index_soot_biofuel"]
        emission_index_soot_electrofuel = self.df["emission_index_soot_electrofuel"]
        emission_index_soot_kerosene = self.df["emission_index_soot_kerosene"]
        emission_index_soot_hydrogen = self.df["emission_index_soot_hydrogen"]

        return (
            emission_index_soot_biofuel,
            emission_index_soot_electrofuel,
            emission_index_soot_kerosene,
            emission_index_soot_hydrogen,
        )


class NonCO2Emissions(AeromapsModel):
    def __init__(self, name="non_co2_emissions", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        emission_index_nox_biofuel: pd.Series = pd.Series(dtype="float64"),
        emission_index_nox_electrofuel: pd.Series = pd.Series(dtype="float64"),
        emission_index_nox_kerosene: pd.Series = pd.Series(dtype="float64"),
        emission_index_nox_hydrogen: pd.Series = pd.Series(dtype="float64"),
        emission_index_soot_biofuel: float = 0.0,
        emission_index_soot_electrofuel: float = 0.0,
        emission_index_soot_kerosene: float = 0.0,
        emission_index_soot_hydrogen: float = 0.0,
        emission_index_h2o_biofuel: float = 0.0,
        emission_index_h2o_electrofuel: float = 0.0,
        emission_index_h2o_kerosene: float = 0.0,
        emission_index_h2o_hydrogen: float = 0.0,
        emission_index_sulfur_biofuel: float = 0.0,
        emission_index_sulfur_electrofuel: float = 0.0,
        emission_index_sulfur_kerosene: float = 0.0,
        emission_index_sulfur_hydrogen: float = 0.0,
        lhv_kerosene: float = 0.0,
        lhv_biofuel: float = 0.0,
        lhv_electrofuel: float = 0.0,
        lhv_hydrogen: float = 0.0,
        energy_consumption_kerosene: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_biofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_electrofuel: pd.Series = pd.Series(dtype="float64"),
        energy_consumption_hydrogen: pd.Series = pd.Series(dtype="float64"),
    ) -> Tuple[pd.Series, pd.Series, pd.Series, pd.Series]:
        """Non-CO2 emissions calculation."""

        self.df["soot_emissions"] = (
            emission_index_soot_biofuel / 10**9 * energy_consumption_biofuel / lhv_biofuel
            + emission_index_soot_electrofuel
            / 10**9
            * energy_consumption_electrofuel
            / lhv_electrofuel
            + emission_index_soot_kerosene / 10**9 * energy_consumption_kerosene / lhv_kerosene
            + emission_index_soot_hydrogen / 10**9 * energy_consumption_hydrogen / lhv_hydrogen
        )
        self.df["h2o_emissions"] = (
            emission_index_h2o_biofuel / 10**9 * energy_consumption_biofuel / lhv_biofuel
            + emission_index_h2o_electrofuel
            / 10**9
            * energy_consumption_electrofuel
            / lhv_electrofuel
            + emission_index_h2o_kerosene / 10**9 * energy_consumption_kerosene / lhv_kerosene
            + emission_index_h2o_hydrogen / 10**9 * energy_consumption_hydrogen / lhv_hydrogen
        )
        self.df["nox_emissions"] = (
            emission_index_nox_biofuel / 10**9 * energy_consumption_biofuel / lhv_biofuel
            + emission_index_nox_electrofuel
            / 10**9
            * energy_consumption_electrofuel
            / lhv_electrofuel
            + emission_index_nox_kerosene / 10**9 * energy_consumption_kerosene / lhv_kerosene
            + emission_index_nox_hydrogen / 10**9 * energy_consumption_hydrogen / lhv_hydrogen
        )
        self.df["sulfur_emissions"] = (
            emission_index_sulfur_biofuel / 10**9 * energy_consumption_biofuel / lhv_biofuel
            + emission_index_sulfur_electrofuel
            / 10**9
            * energy_consumption_electrofuel
            / lhv_electrofuel
            + emission_index_sulfur_kerosene / 10**9 * energy_consumption_kerosene / lhv_kerosene
            + emission_index_sulfur_hydrogen / 10**9 * energy_consumption_hydrogen / lhv_hydrogen
        )

        soot_emissions = self.df["soot_emissions"]
        h2o_emissions = self.df["h2o_emissions"]
        nox_emissions = self.df["nox_emissions"]
        sulfur_emissions = self.df["sulfur_emissions"]

        return soot_emissions, h2o_emissions, nox_emissions, sulfur_emissions



import matplotlib.pyplot as plt
from .constants import plot_3_x, plot_3_y


class RevenuePassengerKilometerPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "rpk"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_rpk,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "rpk"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of the Revenue\nPassenger Kilometer (RPK) for air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Revenue Passenger Kilometer [RPK]")
        self.ax.legend()
        self.ax = plt.gca()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_rpk.set_ydata(self.df.loc[self.prospective_years, "rpk"])

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class RevenueTonneKilometerPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "rtk"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_rtk,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "rtk"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of the Revenue\nTonne Kilometer (RTK) for air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Revenue Tonne Kilometer [RTK]")
        self.ax.legend()
        self.ax = plt.gca()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_rtk.set_ydata(self.df.loc[self.prospective_years, "rtk"])

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class AvailableSeatKilometerPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "ask"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_ask,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "ask"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of the Available\nSeat Kilometer (ASK) for air transport")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Available Seat Kilometer [ASK]")
        self.ax.legend()
        self.ax = plt.gca()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_ask.set_ydata(self.df.loc[self.prospective_years, "ask"])

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class TotalAircraftDistancePlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "total_aircraft_distance"] / 10**9,
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_total_aircraft_distance,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "total_aircraft_distance"] / 10**9,
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of the total distance\ntravelled by aircraft")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Total distance travelled by aircraft [in billions of km]")
        self.ax.legend()
        self.ax = plt.gca()
        self.ax.set_xlim(2000, 2050)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_total_aircraft_distance.set_ydata(
            self.df.loc[self.prospective_years, "total_aircraft_distance"] / 10**9
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()



import matplotlib.pyplot as plt
import numpy as np
from .constants import plot_3_x, plot_3_y


class MeanCO2PerRPKPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "co2_emissions_per_rpk"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_emissions_per_rpk,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "co2_emissions_per_rpk"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of CO2 emissions\nper passenger and per kilometer")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("CO2 emissions per RPK [gCO2/RPK]")
        self.ax.legend(loc=0, fontsize=10)
        self.ax.set_xlim(2000, 2050)
        self.ax.set_ylim(
            0,
        )

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_emissions_per_rpk.set_ydata(
            self.df.loc[self.prospective_years, "co2_emissions_per_rpk"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class MeanCO2PerRTKPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.historic_years,
            self.df.loc[self.historic_years, "co2_emissions_per_rtk"],
            color="black",
            linestyle="-",
            label="History",
            linewidth=2,
        )

        (self.line_emissions_per_rtk,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "co2_emissions_per_rtk"],
            color="blue",
            linestyle="-",
            label="Projections",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title("Evolution of CO2 emissions\nper tonne and per kilometer")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("CO2 emissions per RTK [gCO2/RTK]")
        self.ax.legend(loc=0, fontsize=10)
        self.ax.set_xlim(2000, 2050)
        self.ax.set_ylim(
            0,
        )

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_emissions_per_rtk.set_ydata(
            self.df.loc[self.prospective_years, "co2_emissions_per_rtk"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class PassengerKayaFactorsPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        (self.line_co2,) = self.ax.plot(
            self.years,
            self.df["co2_emissions_passenger"] / self.df.loc[2000, "co2_emissions_passenger"],
            color="blue",
            linestyle="-",
            label="CO2",
            linewidth=3,
        )

        (self.line_rpk,) = self.ax.plot(
            self.years,
            self.df["rpk"] / self.df.loc[2000, "rpk"],
            color="red",
            linestyle="--",
            label="RPK",
            linewidth=2,
        )

        (self.line_energy_per_rpk,) = self.ax.plot(
            self.years,
            self.df["energy_per_ask_mean"]
            / self.df["load_factor"]
            / (self.df.loc[2000, "energy_per_ask_mean"] / self.df.loc[2000, "load_factor"]),
            color="gold",
            linestyle="--",
            label="E/ASK",
            linewidth=2,
        )

        (self.line_co2_per_energy,) = self.ax.plot(
            self.years,
            self.df["co2_per_energy_mean"] / self.df.loc[2000, "co2_per_energy_mean"],
            color="green",
            linestyle="--",
            label="CO2/E",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title(
            "Evolution of the factors in the Kaya equation\nfor passenger air transport (historical until 2019)"
        )
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Reference to 2000 with logarithmic scale")
        self.ax.legend()
        ax = plt.gca()
        self.ax.set_xlim(2000, 2050)
        self.ax.set_yscale("log")

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_co2.set_ydata(
            self.df["co2_emissions_passenger"] / self.df.loc[2000, "co2_emissions_passenger"]
        )

        self.line_rpk.set_ydata(self.df["rpk"] / self.df.loc[2000, "rpk"])

        self.line_energy_per_rpk.set_ydata(
            self.df["energy_per_ask_mean"]
            / self.df["load_factor"]
            / (self.df.loc[2000, "energy_per_ask_mean"] / self.df.loc[2000, "load_factor"])
        )

        self.line_co2_per_energy.set_ydata(
            self.df["co2_per_energy_mean"] / self.df.loc[2000, "co2_per_energy_mean"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class FreightKayaFactorsPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        (self.line_co2,) = self.ax.plot(
            self.years,
            self.df["co2_emissions_freight"] / self.df.loc[2000, "co2_emissions_freight"],
            color="blue",
            linestyle="-",
            label="CO2",
            linewidth=3,
        )

        (self.line_rtk,) = self.ax.plot(
            self.years,
            self.df["rtk"] / self.df.loc[2000, "rtk"],
            color="red",
            linestyle="--",
            label="RTK",
            linewidth=2,
        )

        (self.line_energy_per_rtk,) = self.ax.plot(
            self.years,
            self.df["energy_per_rtk_mean"] / self.df.loc[2000, "energy_per_rtk_mean"],
            color="gold",
            linestyle="--",
            label="E/RTK",
            linewidth=2,
        )

        (self.line_co2_per_energy,) = self.ax.plot(
            self.years,
            self.df["co2_per_energy_mean"] / self.df.loc[2000, "co2_per_energy_mean"],
            color="green",
            linestyle="--",
            label="CO2/E",
            linewidth=2,
        )

        self.ax.grid()
        self.ax.set_title(
            "Evolution of the factors in the Kaya equation\nfor freight air transport (historical until 2019)"
        )
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Reference to 2000 with logarithmic scale")
        self.ax.legend()
        ax = plt.gca()
        self.ax.set_xlim(2000, 2050)
        self.ax.set_yscale("log")

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.line_co2.set_ydata(
            self.df["co2_emissions_freight"] / self.df.loc[2000, "co2_emissions_freight"]
        )

        self.line_rtk.set_ydata(self.df["rtk"] / self.df.loc[2000, "rtk"])

        self.line_energy_per_rtk.set_ydata(
            self.df["energy_per_rtk_mean"] / self.df.loc[2000, "energy_per_rtk_mean"]
        )

        self.line_co2_per_energy.set_ydata(
            self.df["co2_per_energy_mean"] / self.df.loc[2000, "co2_per_energy_mean"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class LeversOfActionDistributionPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        if (
            self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
            - self.df.loc[2050, "cumulative_co2_emissions"]
            > 0
        ):
            if (
                self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                - self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                >= 0
            ):
                sizes = [
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                ]
                sizes = np.round(sizes, 3)
            else:
                sizes = [
                    0,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                ]
                sizes = np.round(sizes, 3)

            labels = (
                "Sufficiency " + str(round(sizes[0], 1)) + "%",
                "Efficiency " + str(round(sizes[1], 1)) + "%",
                "Energy " + str(round(sizes[2], 1)) + "%",
                "Others " + str(round(sizes[3], 1)) + "%",
            )
            colors = "red", "gold", "yellowgreen", "lightskyblue"

            self.line_part_reduction_CO2 = self.ax.pie(
                sizes,
                colors=colors,
                shadow=False,
                startangle=90,
                counterclock=False,
            )

            self.ax.set_title(
                "Impact of the levers of action\non the reduction of cumulative\nCO2 emissions compared to the"
                "\nbaseline scenario"
            )

            self.ax.legend(labels, loc="best")

            self.fig.canvas.header_visible = False
            self.fig.canvas.toolbar_position = "bottom"
            # self.fig.canvas.layout.width = "auto"
            # self.fig.canvas.layout.height = "auto"
            self.fig.tight_layout()

        else:
            sizes = [1, 0, 0, 0]

            labels = ("", "Non-displayable graph           ", "    with these settings        ", "")
            colors = "white", "white", "white", "white"

            self.line_part_reduction_CO2 = self.ax.pie(
                sizes,
                colors=colors,
                shadow=False,
                startangle=90,
                counterclock=False,
            )

            self.ax.set_title(
                "Impact of the levers of action\non the reduction of cumulative\nCO2 emissions compared to the"
                "\nbaseline scenario"
            )

            self.ax.legend(labels, loc="center")

            self.fig.canvas.header_visible = False
            self.fig.canvas.toolbar_position = "bottom"
            # self.fig.canvas.layout.width = "auto"
            # self.fig.canvas.layout.height = "auto"
            self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        if (
            self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
            - self.df.loc[2050, "cumulative_co2_emissions"]
            > 0
        ):

            if (
                self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                - self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                >= 0
            ):
                sizes = [
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology_baseline3"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                ]
                sizes = np.round(sizes, 3)
            else:
                sizes = [
                    0,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_load_factor"]
                        - self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                    (
                        self.df.loc[2050, "cumulative_co2_emissions_including_energy"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    / (
                        self.df.loc[2050, "cumulative_co2_emissions_2019technology"]
                        - self.df.loc[2050, "cumulative_co2_emissions"]
                    )
                    * 100,
                ]
                sizes = np.round(sizes, 3)

            self.ax.clear()

            labels = (
                "Sufficiency " + str(round(sizes[0], 1)) + "%",
                "Efficiency " + str(round(sizes[1], 1)) + "%",
                "Energy " + str(round(sizes[2], 1)) + "%",
                "Others " + str(round(sizes[3], 1)) + "%",
            )
            colors = "red", "gold", "yellowgreen", "lightskyblue"

            self.line_part_reduction_CO2 = self.ax.pie(
                sizes,
                colors=colors,
                shadow=False,
                startangle=90,
                counterclock=False,
            )

            self.ax.set_title(
                "Impact of the levers of action\non the reduction of cumulative\nCO2 emissions compared to the"
                "\nbaseline scenario"
            )

            self.ax.legend(labels, loc="best")

            self.ax.collections.clear()
            self.ax.relim()
            self.ax.autoscale_view()
            self.fig.canvas.draw()

        else:

            sizes = [1, 0, 0, 0]

            self.ax.clear()

            labels = ("", "Non-displayable graph           ", "    with these settings        ", "")
            colors = "white", "white", "white", "white"

            self.line_part_reduction_CO2 = self.ax.pie(
                sizes,
                colors=colors,
                shadow=False,
                startangle=90,
                counterclock=False,
            )

            self.ax.set_title(
                "Impact of the levers of action\non the reduction of cumulative\nCO2 emissions compared to the"
                "\nbaseline scenario"
            )

            self.ax.legend(labels, loc="center")

            self.ax.collections.clear()
            self.ax.relim()
            self.ax.autoscale_view()
            self.fig.canvas.draw()



from aeromaps.models.parameters import AllParameters
from aeromaps.models.impacts.emissions.non_co2_emissions import NonCO2Emissions
from aeromaps.models.impacts.effective_radiative_forcing.effective_radiative_forcing import (
    ERF,
)
from aeromaps.models.impacts.effective_radiative_forcing.effective_radiative_forcing import (
    DetailedERF,
)
from aeromaps.core.gemseo import AeromapsModelWrapper
from gemseo.api import create_discipline, generate_n2_plot

disc1 = AeromapsModelWrapper(model=NonCO2Emissions(parameters=AllParameters()))
disc2 = AeromapsModelWrapper(model=ERF(parameters=AllParameters()))
disc3 = AeromapsModelWrapper(model=DetailedERF(parameters=AllParameters()))

disciplines = [disc1, disc2, disc3]

generate_n2_plot(disciplines)

process = create_discipline("MDOChain", disciplines=disciplines)

outputs = process.execute()

print(outputs)



import os.path as pth
from json import dump
from dataclasses import fields
import numpy as np
import pandas as pd

pd.options.display.max_rows = 150
pd.set_option("display.max_columns", 150)
pd.set_option("max_colwidth", 200)
pd.options.mode.chained_assignment = None

from gemseo.core.discipline import MDODiscipline
from gemseo import generate_n2_plot, create_mda


from aeromaps.core.gemseo import AeromapsModelWrapper
from aeromaps.core.models import models_simple
from aeromaps.models.parameters import all_parameters
from aeromaps.utils.functions import _dict_to_df
from aeromaps.plots import available_plots
from aeromaps.models.air_transport.aircraft_fleet_and_operations.fleet.fleet_model import (
    Fleet,
    FleetModel,
)


DATA_FOLDER = pth.join(pth.dirname(__file__), "..", "resources", "data")

# Contains all data (parameters, inputs, outputs)
EXCEL_DATA_FILE = pth.join(DATA_FOLDER, "data.xlsx")
EXCEL_DATA_INFORMATION_FILE = pth.join(DATA_FOLDER, "data_information.csv")
# Contains parameters and inputs
PARAMETERS_JSON_DATA_FILE = pth.join(DATA_FOLDER, "parameters.json")
# Contains outputs
OUTPUTS_JSON_DATA_FILE = pth.join(DATA_FOLDER, "outputs.json")


class AeromapsProcess(object):
    def __init__(
        self,
        models=models_simple,
        parameters=all_parameters,
        read_json=False,
        fleet=False,
        add_examples_aircraft_and_subcategory=True,
    ):
        self.models = models
        self.parameters = parameters
        self.disciplines = []
        self.data = {}
        self.json = {}

        # Read parameters
        if read_json:
            self.parameters = self.parameters.read_json(file_name=PARAMETERS_JSON_DATA_FILE)

        self._initialize_years()

        self._initialize_disciplines(
            fleet, add_examples_aircraft_and_subcategory=add_examples_aircraft_and_subcategory
        )

        # Create GEMSEO process
        self.process = create_mda(
            "MDAChain", disciplines=self.disciplines, grammar_type=MDODiscipline.GrammarType.SIMPLE
        )

        self._initialize_data()

        self._update_variables()

    def _initialize_data(self):
        # Inputs
        self.data["float_inputs"] = {}
        # TODO: explore the possibility of using a dataframe for vector inputs
        self.data["vector_inputs"] = {}

        # Outputs
        self.data["float_outputs"] = {}
        self.data["vector_outputs"] = pd.DataFrame(index=self.data["years"]["full_years"])

    def _initialize_disciplines(self, fleet, add_examples_aircraft_and_subcategory=True):
        for name, model in self.models.items():
            # TODO: check how to avoid providing all parameters
            model.parameters = self.parameters
            model._initialize_df()
            if hasattr(model, "compute"):
                model = AeromapsModelWrapper(model=model)
                self.disciplines.append(model)
            else:
                print(model.name)
        if fleet:
            self.fleet = Fleet(
                add_examples_aircraft_and_subcategory=add_examples_aircraft_and_subcategory
            )
            self.fleet_model = FleetModel(fleet=self.fleet)
            self.fleet_model.parameters = self.parameters
            self.fleet_model._initialize_df()
            self.models["passenger_aircraft_efficiency_complex"].fleet_model = self.fleet_model
            self.models["passenger_aircraft_doc_non_energy_complex"].fleet_model = self.fleet_model
        else:
            self.fleet = None

    def _initialize_years(self):
        # Years
        self.data["years"] = {}
        self.data["years"]["full_years"] = list(
            range(self.parameters.historic_start_year, self.parameters.end_year + 1)
        )
        self.data["years"]["historic_years"] = list(
            range(
                self.parameters.historic_start_year,
                self.parameters.prospection_start_year,
            )
        )
        self.data["years"]["prospective_years"] = list(
            range(self.parameters.prospection_start_year - 1, self.parameters.end_year + 1)
        )

    def compute(self):

        if self.fleet is not None:
            self.fleet_model.compute()
            self.models["passenger_aircraft_efficiency_complex"].fleet_model = self.fleet_model
            self.models["passenger_aircraft_doc_non_energy_complex"].fleet_model = self.fleet_model

        input_data = self._set_inputs()

        if self.fleet is not None:
            # This is needed since fleet model is particular discipline
            input_data["dummy_fleet_model_output"] = np.random.rand(1, 1)

        self.process.execute(input_data=input_data)

        self._update_variables()

        self.write_json()

    def write_json(self, file_name=OUTPUTS_JSON_DATA_FILE):
        with open(file_name, "w", encoding="utf-8") as f:
            dump(self.json, f, ensure_ascii=False, indent=4)

    def write_excel(self, file_name=EXCEL_DATA_FILE):

        with pd.ExcelWriter(file_name) as writer:
            self.data_information_df.to_excel(writer, sheet_name="Data Information")
            self.vector_inputs_df.to_excel(writer, sheet_name="Vector Inputs")
            self.float_inputs_df.to_excel(writer, sheet_name="Float Inputs")
            self.vector_outputs_df.to_excel(writer, sheet_name="Vector Outputs")
            self.float_outputs_df.to_excel(writer, sheet_name="Float Outputs")

    def generate_n2(self):
        generate_n2_plot(self.disciplines)

    def update_parameters(self):
        for name, model in self.models.items():
            model.parameters = self.parameters

    def list_available_plots(self):
        return list(available_plots.keys())

    def list_float_inputs(self):
        return self.data["float_inputs"]

    def plot(self, name, save=False):

        if name in available_plots:
            fig = available_plots[name](self.data)
            if save:
                fig.fig.savefig(f"{name}.pdf")
        else:
            raise NameError(
                f"Plot {name} is not available. List of available plots: {list(available_plots.keys())}"
            )
        return fig

    def _set_inputs(self):

        all_inputs = {}
        self._format_input_vectors()
        # TODO: make this more efficient
        for disc in self.disciplines:
            disc.model.parameters = self.parameters
            disc.model._initialize_df()
            disc.update_defaults()
            all_inputs.update(disc.default_inputs)

        return all_inputs

    def _format_input_vectors(self):
        for field in fields(self.parameters):
            field_name = field.name
            field_value = getattr(self.parameters, field_name)
            if not isinstance(field_value, (float, int)):
                new_size = self.parameters.end_year - self.parameters.historic_start_year + 1
                new_value = np.pad(
                    field_value,
                    (0, new_size - field_value.size),
                    mode="constant",
                    constant_values=np.nan,
                )
                setattr(self.parameters, field_name, new_value)

    def _update_variables(self):

        self._update_data_from_model()

        self._update_dataframes_from_data()

        self._update_json_from_data()

    def _update_data_from_model(self):

        # Inputs
        all_inputs = self.process.get_input_data_names()

        for name in all_inputs:
            try:
                value = getattr(self.parameters, name)
                if isinstance(value, (float, int)):
                    self.data["float_inputs"][name] = value
                else:
                    new_values = []
                    for val in value:
                        if not np.isnan(val):
                            new_values.append(val)
                    self.data["vector_inputs"][name] = new_values
            except:
                pass

        # Outputs
        if self.data["vector_outputs"].columns.size == 0:
            first_computation = True
        else:
            first_computation = False

        # TODO: better to use _local_data?
        for disc in self.disciplines:
            if hasattr(disc.model, "df") and disc.model.df.columns.size != 0:
                if first_computation:
                    self.data["vector_outputs"] = pd.concat(
                        [self.data["vector_outputs"], disc.model.df], axis=1
                    )
                else:

                    self.data["vector_outputs"].update(disc.model.df)

                    # self.data["vector_outputs"] = self.data["vector_outputs"].merge(disc.model.df)
            self.data["float_outputs"].update(disc.model.float_outputs)

    def _update_dataframes_from_data(self):

        # Float parameters
        data = {
            "Name": self.data["float_inputs"].keys(),
            "Value": self.data["float_inputs"].values(),
        }
        self.float_inputs_df = pd.DataFrame(data=data)

        # Vector parameters
        self.vector_inputs_df = _dict_to_df(self.data["vector_inputs"], orient="columns")
        self.vector_inputs_df.sort_index(axis=1, inplace=True)

        # Float outputs df
        data = {
            "Name": self.data["float_outputs"].keys(),
            "Value": self.data["float_outputs"].values(),
        }
        self.float_outputs_df = pd.DataFrame(data=data)

        # Vector outputs dataframe
        self.vector_outputs_df = self.data["vector_outputs"]
        self.vector_outputs_df.sort_index(axis=1, inplace=True)

        # Variable information
        self._read_data_information()

    def _update_json_from_data(self):
        def convert_values_from_array_to_list(d):
            for key, value in d.items():
                if isinstance(value, (pd.Series, np.ndarray)):
                    d[key] = list(value)
            return d

        # Float inputs
        self.json["float_inputs"] = convert_values_from_array_to_list(self.data["float_inputs"])

        # Vector inputs
        self.json["vector_inputs"] = convert_values_from_array_to_list(self.data["vector_inputs"])

        # Float outputs
        self.json["float_outputs"] = convert_values_from_array_to_list(self.data["float_outputs"])

        # Vector outputs
        self.json["vector_outputs"] = convert_values_from_array_to_list(
            self.data["vector_outputs"].to_dict("list")
        )

    def _read_data_information(self):
        df = pd.read_csv(EXCEL_DATA_INFORMATION_FILE, encoding="utf-8", sep=";")

        var_infos_df = pd.DataFrame()
        for data_type, variables in self.data.items():
            for variable in variables:
                # If the variable exists in the csv we extract the information
                if variable in df["Name"].values:

                    data = df.loc[df["Name"] == variable]
                    data["Type"] = data_type
                    var_infos_df = pd.concat([var_infos_df, data], ignore_index=True)

                # If not we create a new one with just the type as information
                else:
                    data = {
                        "Name": [variable],
                        "Type": [data_type],
                        "Unit": ["N/A"],
                        "Description": ["N/A"],
                    }
                    new_row = pd.DataFrame(data=data)
                    var_infos_df = pd.concat([var_infos_df, new_row], ignore_index=True)

        self.data_information_df = var_infos_df



# @Time : 13/03/2023 15:14
# @Author : a.salgas
# @File : abatement_potential.py
# @Software: PyCharm

from typing import Tuple

import numpy as np
import pandas as pd

from aeromaps.models.base import AeromapsModel


class BiofuelAbatementPotential(AeromapsModel):
    def __init__(self, name="biofuel_abatement_potential", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        biofuel_atj_efficiency: pd.Series = pd.Series(dtype="float64"),
        # biomass_atj_consumption: pd.Series = pd.Series(dtype="float64"),
        kerosene_emission_factor: pd.Series = pd.Series(dtype="float64"),
        biofuel_atj_emission_factor: pd.Series = pd.Series(dtype="float64"),
        # biomass_hefa_fog_consumption: pd.Series = pd.Series(dtype="float64"),
        biofuel_hefa_fuel_efficiency: pd.Series = pd.Series(dtype="float64"),
        biofuel_hefa_fog_emission_factor: pd.Series = pd.Series(dtype="float64"),
        # biomass_hefa_others_consumption: pd.Series = pd.Series(dtype="float64"),
        biofuel_hefa_oil_efficiency: pd.Series = pd.Series(dtype="float64"),
        biofuel_hefa_others_emission_factor: pd.Series = pd.Series(dtype="float64"),
        # biomass_ft_consumption: pd.Series = pd.Series(dtype="float64"),
        biofuel_ft_efficiency: pd.Series = pd.Series(dtype="float64"),
        biofuel_ft_msw_emission_factor: pd.Series = pd.Series(dtype="float64"),
        # biofuel_ft_msw_share: pd.Series = pd.Series(dtype="float64"),
        # biofuel_ft_others_share: pd.Series = pd.Series(dtype="float64"),
        biofuel_ft_others_emission_factor: pd.Series = pd.Series(dtype="float64"),
        # hydrogen_electrolysis_emission_factor: pd.Series = pd.Series(dtype="float64"),
        # hydrogen_replacement_ratio: float = 1.0,
        electricity_hydrogen_consumption: pd.Series = pd.Series(dtype="float64"),
        electrolysis_efficiency: pd.Series = pd.Series(dtype="float64"),
        # liquefaction_efficiency: pd.Series = pd.Series(dtype="float64"),
        electricity_electrofuel_consumption: pd.Series = pd.Series(dtype="float64"),
        electrofuel_hydrogen_efficiency: pd.Series = pd.Series(dtype="float64"),
        electrofuel_emission_factor: pd.Series = pd.Series(dtype="float64"),
        available_biomass_atj: float = 0.0,
        available_biomass_hefa_fog: float = 0.0,
        available_biomass_hefa_others: float = 0.0,
        available_biomass_ft_msw: float = 0.0,
        available_biomass_ft_others: float = 0.0,
        aviation_biomass_allocated_share: float = 0.0,
        aviation_available_electricity: float = 0.0,
    ) -> Tuple[
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
    ]:
        """Maximal and effective abatement potential through biofuel usage under the allocated biomass hypothesis."""

        ##### Maximal #####

        energy_avail_atj = (
            aviation_biomass_allocated_share
            / 100
            * available_biomass_atj
            * 10**12
            * biofuel_atj_efficiency
        )

        abatement_potential_atj = (
            energy_avail_atj * (kerosene_emission_factor - biofuel_atj_emission_factor) / 1000000
        )

        # Biomass consumption in in EJ, while emission factors are in gCO2e/MJ, abatement potential is in tCO2e

        energy_avail_hefa_fog = (
            aviation_biomass_allocated_share
            / 100
            * available_biomass_hefa_fog
            * 10**12
            * biofuel_hefa_fuel_efficiency
        )
        abatement_potential_hefa_fog = (
            energy_avail_hefa_fog
            * (kerosene_emission_factor - biofuel_hefa_fog_emission_factor)
            / 1000000
        )

        energy_avail_hefa_others = (
            aviation_biomass_allocated_share
            / 100
            * available_biomass_hefa_others
            * 10**12
            * biofuel_hefa_oil_efficiency
            * biofuel_hefa_fuel_efficiency
        )
        abatement_potential_hefa_others = (
            energy_avail_hefa_others
            * (kerosene_emission_factor - biofuel_hefa_others_emission_factor)
            / 1000000
        )

        energy_avail_ft_msw = (
            aviation_biomass_allocated_share
            / 100
            * available_biomass_ft_msw
            * 10**12
            * biofuel_ft_efficiency
        )
        abatement_potential_ft_msw = (
            energy_avail_ft_msw
            * (kerosene_emission_factor - biofuel_ft_msw_emission_factor)
            / 1000000
        )

        energy_avail_ft_others = (
            aviation_biomass_allocated_share
            / 100
            * available_biomass_ft_others
            * 10**12
            * biofuel_ft_efficiency
        )
        abatement_potential_ft_others = (
            energy_avail_ft_others
            * (kerosene_emission_factor - biofuel_ft_others_emission_factor)
            / 1000000
        )

        ## TODO reactivate for potential hydrogen MACC
        # h2_avoided_emissions_factor = (
        #     kerosene_emission_factor / hydrogen_replacement_ratio
        #     - hydrogen_electrolysis_emission_factor
        # )
        # energy_avail_hydrogen_electrolysis = (
        #     aviation_available_electricity
        #     * electricity_hydrogen_consumption
        #     / (electricity_electrofuel_consumption + electricity_hydrogen_consumption)
        #     * 10**12
        #     * electrolysis_efficiency
        #     * liquefaction_efficiency
        # )
        # abatement_potential_hydrogen_electrolysis = (
        #     energy_avail_hydrogen_electrolysis * h2_avoided_emissions_factor / 1000000
        # )

        ## The same electricity consumption share (hydrogen/electrofuel) is kept for maxiaml abatement potential computation

        energy_avail_electrofuel = (
            aviation_available_electricity
            * electricity_electrofuel_consumption
            / (electricity_electrofuel_consumption + electricity_hydrogen_consumption)
            * 10**12
            * electrolysis_efficiency
            * electrofuel_hydrogen_efficiency
        )
        abatement_potential_electrofuel = (
            energy_avail_electrofuel
            * (kerosene_emission_factor - electrofuel_emission_factor)
            / 1000000
        )

        self.df.loc[:, "abatement_potential_atj"] = abatement_potential_atj
        self.df.loc[:, "abatement_potential_hefa_fog"] = abatement_potential_hefa_fog
        self.df.loc[:, "abatement_potential_hefa_others"] = abatement_potential_hefa_others
        self.df.loc[:, "abatement_potential_ft_msw"] = abatement_potential_ft_msw
        self.df.loc[:, "abatement_potential_ft_others"] = abatement_potential_ft_others
        # self.df.loc[
        #     :, "abatement_potential_hydrogen_electrolysis"
        # ] = abatement_potential_hydrogen_electrolysis
        self.df.loc[:, "abatement_potential_electrofuel"] = abatement_potential_electrofuel
        self.df.loc[:, "energy_avail_atj"] = energy_avail_atj
        self.df.loc[:, "energy_avail_hefa_fog"] = energy_avail_hefa_fog
        self.df.loc[:, "energy_avail_hefa_others"] = energy_avail_hefa_others
        self.df.loc[:, "energy_avail_ft_msw"] = energy_avail_ft_msw
        self.df.loc[:, "energy_avail_ft_others"] = energy_avail_ft_others
        # self.df.loc[:, "energy_avail_hydrogen_electrolysis"] = energy_avail_hydrogen_electrolysis
        self.df.loc[:, "energy_avail_electrofuel"] = energy_avail_electrofuel

        ##### Effective #####
        # TODO delete this part?

        # abatement_effective_atj = (
        #     biomass_atj_consumption
        #     * 10**12
        #     * biofuel_atj_efficiency
        #     * (kerosene_emission_factor - biofuel_atj_emission_factor)
        #     / 1000000
        # )
        #
        # # Biomass consumption in in EJ, while emission factors are in gCO2e/MJ, abatement effective is in tCO2e
        #
        # abatement_effective_hefa_fog = (
        #     biomass_hefa_fog_consumption
        #     * 10**12
        #     * biofuel_hefa_fuel_efficiency
        #     * (kerosene_emission_factor - biofuel_hefa_fog_emission_factor)
        #     / 1000000
        # )
        #
        # abatement_effective_hefa_others = (
        #     biomass_hefa_others_consumption
        #     * 10**12
        #     * biofuel_hefa_oil_efficiency
        #     * biofuel_hefa_fuel_efficiency
        #     * (kerosene_emission_factor - biofuel_hefa_others_emission_factor)
        #     / 1000000
        # )
        #
        # abatement_effective_ft_msw = (
        #     biomass_ft_consumption
        #     * (biofuel_ft_msw_share / (biofuel_ft_others_share + biofuel_ft_msw_share))
        #     * 10**12
        #     * biofuel_ft_efficiency
        #     * (kerosene_emission_factor - biofuel_ft_msw_emission_factor)
        #     / 1000000
        # )
        #
        # abatement_effective_ft_others = (
        #     biomass_ft_consumption
        #     * (biofuel_ft_others_share / (biofuel_ft_others_share + biofuel_ft_msw_share))
        #     * 10**12
        #     * biofuel_ft_efficiency
        #     * (kerosene_emission_factor - biofuel_ft_others_emission_factor)
        #     / 1000000
        # )
        #
        # h2_avoided_emissions_factor = (
        #     kerosene_emission_factor / hydrogen_replacement_ratio
        #     - hydrogen_electrolysis_emission_factor
        # )
        #
        # abatement_effective_hydrogen_electrolysis = (
        #     electricity_hydrogen_consumption
        #     * 10**12
        #     * electrolysis_efficiency
        #     * liquefaction_efficiency
        #     * h2_avoided_emissions_factor
        #     / 1000000
        # )
        #
        # abatement_effective_electrofuel = (
        #     electricity_electrofuel_consumption
        #     * 10**12
        #     * electrolysis_efficiency
        #     * electrofuel_hydrogen_efficiency
        #     * (kerosene_emission_factor - electrofuel_emission_factor)
        #     / 1000000
        # )

        # TODO delete?
        # self.df.loc[:, "abatement_effective_atj"] = abatement_effective_atj
        # self.df.loc[:, "abatement_effective_hefa_fog"] = abatement_effective_hefa_fog
        # self.df.loc[:, "abatement_effective_hefa_others"] = abatement_effective_hefa_others
        # self.df.loc[:, "abatement_effective_ft_msw"] = abatement_effective_ft_msw
        # self.df.loc[:, "abatement_effective_ft_others"] = abatement_effective_ft_others
        # self.df.loc[
        #     :, "abatement_effective_hydrogen_electrolysis"
        # ] = abatement_effective_hydrogen_electrolysis
        # self.df.loc[:, "abatement_effective_electrofuel"] = abatement_effective_electrofuel

        return (
            abatement_potential_atj,
            abatement_potential_hefa_fog,
            abatement_potential_hefa_others,
            abatement_potential_ft_msw,
            abatement_potential_ft_others,
            # abatement_potential_hydrogen_electrolysis,
            abatement_potential_electrofuel,
            # abatement_effective_atj,
            # abatement_effective_hefa_fog,
            # abatement_effective_hefa_others,
            # abatement_effective_ft_msw,
            # abatement_effective_ft_others,
            # abatement_effective_hydrogen_electrolysis,
            # abatement_effective_electrofuel,
            energy_avail_atj,
            energy_avail_hefa_fog,
            energy_avail_hefa_others,
            energy_avail_ft_msw,
            energy_avail_ft_others,
            # energy_avail_hydrogen_electrolysis,
            energy_avail_electrofuel,
        )



from json import load
import pandas as pd


def _dict_from_json(file_name="parameters.json") -> dict:
    with open(file_name, "r", encoding="utf-8") as f:
        parameters_dict = load(f)

    for key, value in parameters_dict.items():
        if isinstance(value, list):
            parameters_dict[key] = pd.Series(value)

    return parameters_dict


def _dict_to_df(data, orient="index") -> pd.DataFrame:
    df = pd.DataFrame.from_dict(data, orient=orient)
    return df



from typing import Tuple
import numpy as np
import pandas as pd

from aeromaps.models.base import AeromapsModel


class EquivalentCO2Emissions(AeromapsModel):
    def __init__(self, name="equivalent_co2_emissions", *args, **kwargs):
        super().__init__(name=name, *args, **kwargs)

    def compute(
        self,
        total_erf: pd.Series = pd.Series(dtype="float64"),
        co2_erf: pd.Series = pd.Series(dtype="float64"),
        co2_emissions: pd.Series = pd.Series(dtype="float64"),
        erf_coefficient_co2: float = 0.0,
    ) -> Tuple[
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
        pd.Series,
    ]:
        """Equivalent CO2 emissions calculation using smoothed data and GWP* method."""

        # Smooth on 5 years (except for extrem values)

        self.df["total_erf_smooth"] = total_erf
        self.df["co2_erf_smooth"] = co2_erf
        self.df["co2_emissions_smooth"] = co2_emissions

        for k in range(self.historic_start_year + 2, self.end_year - 1):
            self.df.loc[k, "total_erf_smooth"] = (
                total_erf.loc[k - 2]
                + total_erf.loc[k - 1]
                + total_erf.loc[k]
                + total_erf.loc[k + 1]
                + total_erf.loc[k + 2]
            ) / 5
            self.df.loc[k, "co2_erf_smooth"] = (
                co2_erf.loc[k - 2]
                + co2_erf.loc[k - 1]
                + co2_erf.loc[k]
                + co2_erf.loc[k + 1]
                + co2_erf.loc[k + 2]
            ) / 5
            self.df.loc[k, "co2_emissions_smooth"] = (
                co2_emissions.loc[k - 2]
                + co2_emissions.loc[k - 1]
                + co2_emissions.loc[k]
                + co2_emissions.loc[k + 1]
                + co2_emissions.loc[k + 2]
            ) / 5

        # Cumulative CO2 smooth emissions(Gtwe)
        self.df.loc[self.prospection_start_year - 1, "cumulative_co2_emissions_smooth"] = 0.0
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "cumulative_co2_emissions_smooth"] = (
                self.df.loc[k - 1, "cumulative_co2_emissions_smooth"]
                + self.df.loc[k, "co2_emissions_smooth"] / 1000
            )

        total_erf_smooth = self.df["total_erf_smooth"]
        co2_erf_smooth = self.df["co2_erf_smooth"]
        co2_emissions_smooth = self.df["co2_emissions_smooth"]
        cumulative_co2_emissions_smooth = self.df["cumulative_co2_emissions_smooth"]

        # DeltaF/Deltat
        for k in range(self.historic_start_year, self.end_year + 1):
            self.df.loc[k, "non_co2_erf_smooth"] = (
                self.df.loc[k, "total_erf_smooth"] - self.df.loc[k, "co2_erf_smooth"]
            )
        gwpstar_variation_duration = 20
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "non_co2_erf_smooth_variation"] = (
                self.df.loc[k, "non_co2_erf_smooth"]
                - self.df.loc[k - gwpstar_variation_duration, "non_co2_erf_smooth"]
            ) / gwpstar_variation_duration
        non_co2_erf_smooth = self.df["non_co2_erf_smooth"]
        non_co2_erf_smooth_variation = self.df["non_co2_erf_smooth_variation"]

        # Non-CO2 equivalent emissions (Mtwe)
        climate_time_frame = 100
        co2_absolute_global_warming_potential = erf_coefficient_co2 * 100 / 1000
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "non_co2_equivalent_emissions"] = (
                self.df.loc[k, "non_co2_erf_smooth_variation"]
                * climate_time_frame
                / co2_absolute_global_warming_potential
            )
        non_co2_equivalent_emissions = self.df["non_co2_equivalent_emissions"]

        # Cumulative non-CO2 equivalent emissions (Gtwe)
        self.df.loc[
            self.prospection_start_year - 1, "cumulative_non_co2_equivalent_emissions"
        ] = 0.0
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "cumulative_non_co2_equivalent_emissions"] = (
                self.df.loc[k - 1, "cumulative_non_co2_equivalent_emissions"]
                + self.df.loc[k, "non_co2_equivalent_emissions"] / 1000
            )
        cumulative_non_co2_equivalent_emissions = self.df["cumulative_non_co2_equivalent_emissions"]

        # Equivalent emissions (Mtwe)
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "total_equivalent_emissions"] = (
                self.df.loc[k, "non_co2_equivalent_emissions"]
                + self.df.loc[k, "co2_emissions_smooth"]
            )
        total_equivalent_emissions = self.df["total_equivalent_emissions"]

        # Cumulative equivalent emissions (Gtwe)
        self.df.loc[self.prospection_start_year - 1, "cumulative_total_equivalent_emissions"] = 0.0
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "cumulative_total_equivalent_emissions"] = (
                self.df.loc[k - 1, "cumulative_total_equivalent_emissions"]
                + self.df.loc[k, "total_equivalent_emissions"] / 1000
            )
        cumulative_total_equivalent_emissions = self.df["cumulative_total_equivalent_emissions"]

        self.cumulative_total_equivalent_emissions_end_year = self.df.loc[
            self.end_year, "cumulative_total_equivalent_emissions"
        ]  # A voir avec Scott

        # Share CO2/non-CO2
        for k in range(self.prospection_start_year, self.end_year + 1):
            self.df.loc[k, "total_co2_equivalent_emissions_ratio"] = (
                self.df.loc[k, "total_equivalent_emissions"]
                / self.df.loc[k, "co2_emissions_smooth"]
            )
        total_co2_equivalent_emissions_ratio = self.df["total_co2_equivalent_emissions_ratio"]

        co2_total_erf_ratio = co2_erf / total_erf * 100
        self.df.loc[:, "co2_total_erf_ratio"] = co2_total_erf_ratio

        return (
            total_erf_smooth,
            co2_erf_smooth,
            co2_emissions_smooth,
            cumulative_co2_emissions_smooth,
            non_co2_erf_smooth,
            non_co2_erf_smooth_variation,
            non_co2_equivalent_emissions,
            cumulative_non_co2_equivalent_emissions,
            total_equivalent_emissions,
            cumulative_total_equivalent_emissions,
            total_co2_equivalent_emissions_ratio,
            co2_total_erf_ratio,
        )



from dataclasses import dataclass
import os.path as pth
import pandas as pd
from aeromaps.utils.functions import _dict_from_json

filename = pth.join(
    pth.dirname(pth.abspath(__file__)), "..", "..", "resources", "data", "parameters.json"
)

data = _dict_from_json(file_name=filename)


@dataclass
class ImpactsParameters(object):
    # CO2 emission factor [gCO2/MJ]
    kerosene_emission_factor_2019: float = 87.5  # Global (direct emissions: 73.2)

    # NOx emission index in 2019 [kg_NOx/kg_fuel]
    emission_index_nox_biofuel_2019: float = 0.01514
    emission_index_nox_electrofuel_2019: float = 0.01514
    emission_index_nox_kerosene_2019: float = 0.01514
    emission_index_nox_hydrogen_2019: float = 0.007

    # Soot emission index in 2019 [kg_BC/kg_fuel]
    emission_index_soot_biofuel_2019: float = 6e-6
    emission_index_soot_electrofuel_2019: float = 6e-6
    emission_index_soot_kerosene_2019: float = 3e-5
    emission_index_soot_hydrogen_2019: float = 0.0

    # Other emission index [kg/kg_fuel]
    emission_index_h2o_biofuel: float = 1.231
    emission_index_h2o_electrofuel: float = 1.231
    emission_index_h2o_kerosene: float = 1.231
    emission_index_h2o_hydrogen: float = 9.0
    emission_index_sulfur_biofuel: float = 0.0
    emission_index_sulfur_electrofuel: float = 0.0
    emission_index_sulfur_kerosene: float = 0.0012
    emission_index_sulfur_hydrogen: float = 0.0

    # LHV [MJ/kg]
    lhv_kerosene: float = 44
    lhv_biofuel: float = 44
    lhv_electrofuel: float = 44
    lhv_hydrogen: float = 120

    # ERF coefficients
    erf_coefficient_soot: float = 100.7
    erf_coefficient_co2: float = 0.88
    erf_coefficient_contrails: float = 1.058e-09
    erf_coefficient_h2o: float = 0.0052
    erf_coefficient_nox: float = 11.55 * 0.304348  # Convertir directement de N à NOx !
    erf_coefficient_sulfur: float = -19.9
    direct_co2_erf_2018_reference: float = 30.184

    # Temperature
    temperature_increase_from_aviation_init: pd.Series = data[
        "temperature_increase_from_aviation_init"
    ]  # [°C] Using Grewe et al. (2021)
    TCRE: float = 0.00045  # [°C/GtCO2]

    ### COSTS ###
    # Hydrogen Cost parameters
    hydrogen_replacement_ratio: float = 1.0

    # Electrolysis CAPEX (€ / (kg/day)
    electrolyser_capex_2020: float = 588.0
    electrolyser_capex_2030: float = 459.0
    electrolyser_capex_2040: float = 433.0
    electrolyser_capex_2050: float = 419.0

    # Electrolysis Opex (€ / (kg/day) /year and €/kg

    electrolyser_fixed_opex_2020: float = 21.3
    electrolyser_fixed_opex_2030: float = 20.57
    electrolyser_fixed_opex_2040: float = 20.25
    electrolyser_fixed_opex_2050: float = 20.04
    electrolyser_var_opex_2020: float = 0.20
    electrolyser_var_opex_2030: float = 0.19
    electrolyser_var_opex_2040: float = 0.18
    electrolyser_var_opex_2050: float = 0.18

    # Deprecated since move to efficiency, might be reactivated
    # Electrolysis Specific Electricity (kWh/kg)
    # electrolyser_specific_electricity_2020: float = 51.0
    # electrolyser_specific_electricity_2030: float = 49.3
    # electrolyser_specific_electricity_2040: float = 48.5
    # electrolyser_specific_electricity_2050: float = 48.0

    # Fossil hydrogen plant load factors (also called availability factors)
    gas_ccs_load_factor: float = 0.95
    gas_load_factor: float = 0.95
    coal_ccs_load_factor: float = 0.9
    coal_load_factor: float = 0.9

    # Define carbon capture and storage efficiencies
    gas_ccs_ccs_efficiency: float = 0.95
    coal_ccs_ccs_efficiency: float = 0.95

    # GAS + CCS CAPEX (€/ (kg/day))
    gas_ccs_eis_capex_2020: float = 1728
    gas_ccs_eis_capex_2030: float = 1728
    gas_ccs_eis_capex_2040: float = 1728
    gas_ccs_eis_capex_2050: float = 1728

    # GAS + CCS OPEX (€/ (kg/day)) / year
    gas_ccs_eis_fixed_opex_2020: float = 1728 * 0.04
    gas_ccs_eis_fixed_opex_2030: float = 1728 * 0.04
    gas_ccs_eis_fixed_opex_2040: float = 1728 * 0.04
    gas_ccs_eis_fixed_opex_2050: float = 1728 * 0.04

    # GAS + CCS efficicency
    gas_ccs_efficiency_2020: float = 0.69
    gas_ccs_efficiency_2030: float = 0.69
    gas_ccs_efficiency_2040: float = 0.69
    gas_ccs_efficiency_2050: float = 0.69

    # GAS + CAPEX (€/ (kg/day))
    gas_eis_capex_2020: float = 917
    gas_eis_capex_2030: float = 917
    gas_eis_capex_2040: float = 917
    gas_eis_capex_2050: float = 917

    # GAS OPEX (€/ (kg/day)) / year
    gas_eis_fixed_opex_2020: float = 917 * 0.047
    gas_eis_fixed_opex_2030: float = 917 * 0.047
    gas_eis_fixed_opex_2040: float = 917 * 0.047
    gas_eis_fixed_opex_2050: float = 917 * 0.047

    # GAS efficicency
    gas_efficiency_2020: float = 0.76
    gas_efficiency_2030: float = 0.76
    gas_efficiency_2040: float = 0.76
    gas_efficiency_2050: float = 0.76

    # Coal + CCS CAPEX (€/ (kg/day))
    coal_ccs_eis_capex_2020: float = 2399
    coal_ccs_eis_capex_2030: float = 2399
    coal_ccs_eis_capex_2040: float = 2399
    coal_ccs_eis_capex_2050: float = 2399

    # Coal + CCS OPEX (€/ (kg/day)) / year
    coal_ccs_eis_fixed_opex_2020: float = 2399 * 0.05
    coal_ccs_eis_fixed_opex_2030: float = 2399 * 0.05
    coal_ccs_eis_fixed_opex_2040: float = 2399 * 0.05
    coal_ccs_eis_fixed_opex_2050: float = 2399 * 0.05

    # Coal + CCS efficicency
    coal_ccs_efficiency_2020: float = 0.58
    coal_ccs_efficiency_2030: float = 0.58
    coal_ccs_efficiency_2040: float = 0.58
    coal_ccs_efficiency_2050: float = 0.58

    # Coal CAPEX (€/ (kg/day))
    coal_eis_capex_2020: float = 2304
    coal_eis_capex_2030: float = 2304
    coal_eis_capex_2040: float = 2304
    coal_eis_capex_2050: float = 2304

    # Coal OPEX (€/ (kg/day)) / year
    coal_eis_fixed_opex_2020: float = 2304 * 0.05
    coal_eis_fixed_opex_2030: float = 2304 * 0.05
    coal_eis_fixed_opex_2040: float = 2304 * 0.05
    coal_eis_fixed_opex_2050: float = 2304 * 0.05

    # Coal efficicency
    coal_efficiency_2020: float = 0.6
    coal_efficiency_2030: float = 0.6
    coal_efficiency_2040: float = 0.6
    coal_efficiency_2050: float = 0.6

    # CCS price (€/kgCO2)
    ccs_cost_2020: float = 0.02
    ccs_cost_2030: float = 0.02
    ccs_cost_2040: float = 0.02
    ccs_cost_2050: float = 0.02

    # Liquefier Capex (€/ (kg/day))
    liquefier_capex_2020: float = 1457.33
    liquefier_capex_2030: float = 1457.33
    liquefier_capex_2040: float = 1457.33
    liquefier_capex_2050: float = 1457.33

    # Hydrogen transport cost ratio (Hoelzen et al.)
    transport_cost_ratio: float = 0.1

    # Deprecated since move to efficiency, might be reactivated
    # Liquefier sp. electricity (kWh/kg)
    # liquefier_specific_electricity_2020: float = 7.54
    # liquefier_specific_electricity_2030: float = 7.54
    # liquefier_specific_electricity_2040: float = 7.54
    # liquefier_specific_electricity_2050: float = 7.54

    # Electrofuel plant capex € / (kg/day)
    electrofuel_capex_2020: float = 2311
    electrofuel_capex_2030: float = 2311
    electrofuel_capex_2040: float = 2311
    electrofuel_capex_2050: float = 2311

    # Electrofuel plant Opex (€ / (kg/day) /year
    electrofuel_fixed_opex_2020: float = 0.0
    electrofuel_fixed_opex_2030: float = 0.0
    electrofuel_fixed_opex_2040: float = 0.0
    electrofuel_fixed_opex_2050: float = 0.0

    # €/kg
    electrofuel_var_opex_2020: float = 0.38
    electrofuel_var_opex_2030: float = 0.38
    electrofuel_var_opex_2040: float = 0.38
    electrofuel_var_opex_2050: float = 0.38

    # Deprecated since move to efficiency, might be reactivated
    # Electrofuel plant specific electricity (kWh/kg)
    # electrofuel_specific_electricity_2020: float = 22.9
    # electrofuel_specific_electricity_2030: float = 22.9
    # electrofuel_specific_electricity_2040: float = 22.9
    # electrofuel_specific_electricity_2050: float = 22.9

    # Electrofuel plant specific co2 (kg/kg)

    electrofuel_specific_co2_2020: float = 4.47
    electrofuel_specific_co2_2030: float = 4.47
    electrofuel_specific_co2_2040: float = 4.47
    electrofuel_specific_co2_2050: float = 4.47

    # Electricity market price (€/kWh)

    electricity_cost_2020: float = 0.08
    electricity_cost_2030: float = 0.08
    electricity_cost_2040: float = 0.08
    electricity_cost_2050: float = 0.08

    # Gas market price (€/kWh)

    gas_cost_2020: float = 0.005
    gas_cost_2030: float = 0.011
    gas_cost_2040: float = 0.010
    gas_cost_2050: float = 0.009

    # Coal market price (€/kWh)
    coal_cost_2020: float = 0.006
    coal_cost_2030: float = 0.0057
    coal_cost_2040: float = 0.0054
    coal_cost_2050: float = 0.0052

    # Electricity load factor
    # todo switch cases for ded renwb/grid

    electricity_load_factor: float = 0.95

    # CO2 DAC market price (€/kg)

    co2_cost_2020: float = 0.225
    co2_cost_2030: float = 0.225
    co2_cost_2040: float = 0.225
    co2_cost_2050: float = 0.225

    # CO2 tax level (€/ton)

    co2_tax_2020: float = 50
    co2_tax_2030: float = 100
    co2_tax_2040: float = 200
    co2_tax_2050: float = 300

    ### Kerosene

    # €/L

    kerosene_price_2020: float = 0.41
    kerosene_price_2030: float = 0.41
    kerosene_price_2040: float = 0.41
    kerosene_price_2050: float = 0.41

    ### Biofuel

    # €/L

    biofuel_hefa_fog_mfsp_2020: float = 0.89
    biofuel_hefa_fog_mfsp_2050: float = 0.89
    biofuel_hefa_others_mfsp_2020: float = 1.08
    biofuel_hefa_others_mfsp_2050: float = 1.08
    biofuel_ft_others_mfsp_2020: float = 0.54 * 1.02 + 0.46 * 1.28
    biofuel_ft_others_mfsp_2050: float = 0.54 * 1.02 + 0.46 * 1.28
    biofuel_ft_msw_mfsp_2020: float = 1.02
    biofuel_ft_msw_mfsp_2050: float = 1.02
    biofuel_atj_mfsp_2020: float = 1.42
    biofuel_atj_mfsp_2050: float = 1.42

    # €/kg/day
    biofuel_hefa_fog_capex_2020: float = 307.0
    biofuel_hefa_fog_capex_2050: float = 307.0
    biofuel_hefa_others_capex_2020: float = 276.0
    biofuel_hefa_others_capex_2050: float = 276.0
    biofuel_ft_others_capex_2020: float = 1601 * 0.54 + 2192 * 0.46
    biofuel_ft_others_capex_2050: float = 1601 * 0.54 + 2192 * 0.46
    biofuel_ft_msw_capex_2020: float = 2056.0
    biofuel_ft_msw_capex_2050: float = 2056.0
    biofuel_atj_capex_2020: float = 1211.0
    biofuel_atj_capex_2050: float = 1212.0

    # Economic analyses
    social_discount_rate: float = 0.03

    # DOC simple initialisation

    doc_non_energy_per_ask_short_range_dropin_fuel_init: float = 0.045
    doc_non_energy_per_ask_medium_range_dropin_fuel_init: float = 0.028
    doc_non_energy_per_ask_long_range_dropin_fuel_init: float = 0.023
    doc_non_energy_per_ask_short_range_dropin_fuel_gain: float = 0.0
    doc_non_energy_per_ask_medium_range_dropin_fuel_gain: float = 0.0
    doc_non_energy_per_ask_long_range_dropin_fuel_gain: float = 0.0
    relative_doc_non_energy_per_ask_hydrogen_wrt_dropin_short_range: float = 1.1
    relative_doc_non_energy_per_ask_hydrogen_wrt_dropin_medium_range: float = 1.1
    relative_doc_non_energy_per_ask_hydrogen_wrt_dropin_long_range: float = 1.1



# @Time : 15/06/2023 09:06
# @Author : a.salgas
# @File : costs.py
# @Software: PyCharm
import warnings

import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import numpy as np
import pandas as pd

from .constants import plot_3_x, plot_3_y


class ScenarioEnergyCapitalPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        # mine
        colors = [
            "#ee9b00",
            "#ffbf47",
            "#bb3e03",
            "#0c9e30",
            "#097223",
            "#828782",
            "#52F752",
            "#0ABAFF",
            "#8CAAB6",
            "#0ABAFF",
            "#8CAAB6",
            "#87AE87",
        ]

        self.annual_energy_invest = self.ax.stackplot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "plant_building_cost_hefa_fog"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_hefa_others"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_atj"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_ft_others"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_ft_msw"].fillna(0),
            self.df.loc[self.prospective_years, "electrofuel_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "electrolysis_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_ccs_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_ccs_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "liquefaction_plant_building_cost"].fillna(0),
            colors=colors,
            lw=0.5,
            edgecolor="black",
        )

        self.ax.grid(axis="y")
        self.ax.set_title("Annual investment per pathway (w/o fossil)")
        self.ax.set_xlabel("Year")
        self.ax.set_xlim(2020, max(self.prospective_years) - 1)
        self.ax.set_ylabel("Annual Capital Investment [M€]")
        # self.ax = plt.gca()

        legend = self.ax.legend(
            [
                "Bio - HEFA FOG",
                "Bio - HEFA Others",
                "Bio - Alcohol to Jet",
                "Bio - FT Others",
                "Bio - FT Municipal Waste",
                "Electrofuel",
                "$H_2$ - Electrolysis",
                "$H_2$ - Gas + CCS",
                "$H_2$ - Gas",
                "$H_2$ - Coal + CCS",
                "$H_2$ - Coal",
                "(L-)$H_2$ (Liquefaction)",
            ],
            loc="upper left",
            prop={"size": 7},
        )

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.ax.clear()

        colors = [
            "#ee9b00",
            "#ffbf47",
            "#bb3e03",
            "#0c9e30",
            "#097223",
            "#828782",
            "#52F752",
            "#0ABAFF",
            "#8CAAB6",
            "#0ABAFF",
            "#8CAAB6",
            "#87AE87",
        ]

        self.annual_energy_invest = self.ax.stackplot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "plant_building_cost_hefa_fog"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_hefa_others"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_atj"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_ft_others"].fillna(0),
            self.df.loc[self.prospective_years, "plant_building_cost_ft_msw"].fillna(0),
            self.df.loc[self.prospective_years, "electrofuel_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "electrolysis_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_ccs_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_ccs_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_plant_building_cost"].fillna(0),
            self.df.loc[self.prospective_years, "liquefaction_plant_building_cost"].fillna(0),
            colors=colors,
            lw=0.5,
            edgecolor="black",
        )

        self.ax.grid(axis="y")
        self.ax.set_title("Annual investment per pathway (w/o fossil)")
        self.ax.set_xlabel("Year")
        self.ax.set_xlim(2020, max(self.prospective_years) - 1)
        self.ax.set_ylabel("Annual Capital Investment [M€]")

        legend = self.ax.legend(
            [
                "Bio - HEFA FOG",
                "Bio - HEFA Others",
                "Bio - Alcohol to Jet",
                "Bio - FT Others",
                "Bio - FT Municipal Waste",
                "Electrofuel",
                "$H_2$ - Electrolysis",
                "$H_2$ - Gas + CCS",
                "$H_2$ - Gas",
                "$H_2$ - Coal + CCS",
                "$H_2$ - Coal",
                "(L-)$H_2$ (Liquefaction)",
            ],
            loc="upper left",
            prop={"size": 7},
        )

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class ScenarioEnergyExpensesPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        # mine
        colors = [
            "#2A3438",
            "#2A3438",
            "#ee9b00",
            "#ee9b00",
            "#ffbf47",
            "#ffbf47",
            "#bb3e03",
            "#bb3e03",
            "#0c9e30",
            "#0c9e30",
            "#097223",
            "#097223",
            "#828782",
            "#828782",
            "#52F752",
            "#52F752",
            "#0ABAFF",
            "#0ABAFF",
            "#8CAAB6",
            "#8CAAB6",
            "#0ABAFF",
            "#0ABAFF",
            "#8CAAB6",
            "#8CAAB6",
            "#87AE87",
        ]

        self.annual_energy_expenses = self.ax.stackplot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "kerosene_cost"].fillna(0),
            self.df.loc[self.prospective_years, "kerosene_carbon_tax_cost"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_hefa_fog"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_fog"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_hefa_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_atj"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_atj"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_ft_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_ft_msw"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_msw"].fillna(0),
            self.df.loc[self.prospective_years, "electrofuel_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "electrofuel_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "electrolysis_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "electrolysis_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "gas_ccs_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_ccs_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "gas_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "coal_ccs_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_ccs_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "coal_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "liquefaction_h2_total_cost"].fillna(0)
            + self.df.loc[self.prospective_years, "transport_h2_total_cost"].fillna(0),
            colors=colors,
            lw=0.5,
            edgecolor="black",
        )

        self.ax.grid(axis="y")
        self.ax.set_title("Annual energy expenses per pathway")
        self.ax.set_ylabel("Energy expenses [M€]")

        primary_legend_entries = [
            "Fossil Kerosene",
            "_nolegend_",
            "Bio - HEFA FOG",
            "_nolegend_",
            "Bio - HEFA Others",
            "_nolegend_",
            "Bio - Alcohol to Jet",
            "_nolegend_",
            "Bio - FT Others",
            "_nolegend_",
            "Bio - FT MSW",
            "_nolegend_",
            "Electrofuel",
            "_nolegend_",
            "Electrolysis $H_2$ ",
            "_nolegend_",
            "Gas CCS $H_2$ ",
            "_nolegend_",
            "Gas $H_2$ ",
            "_nolegend_",
            "Coal CCS $H_2$ ",
            "_nolegend_",
            "Coal $H_2$ ",
            "_nolegend_",
            "$H_2$ liq. & transport",
        ]

        stacks = self.annual_energy_expenses

        hatches = [
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
        ]
        for stack, hatch in zip(stacks, hatches):
            stack.set_hatch(hatch)

        self.ax.set_xlim(2020, 2050)

        warnings.filterwarnings("ignore")

        primary_legend = self.ax.legend(primary_legend_entries, loc="upper left", prop={"size": 7})
        self.ax.add_artist(primary_legend)

        warnings.resetwarnings()

        # Create hatch legend manually
        hatch_patch = mpatches.Patch(facecolor="white", hatch="||", edgecolor="black")
        self.ax.legend(
            handles=[hatch_patch], labels=["Carbon Tax"], loc="upper right", prop={"size": 7}
        )

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, df_data):

        self.df = df_data["vector_outputs"]

        self.ax.clear()

        colors = [
            "#2A3438",
            "#2A3438",
            "#ee9b00",
            "#ee9b00",
            "#ffbf47",
            "#ffbf47",
            "#bb3e03",
            "#bb3e03",
            "#0c9e30",
            "#0c9e30",
            "#097223",
            "#097223",
            "#828782",
            "#828782",
            "#52F752",
            "#52F752",
            "#0ABAFF",
            "#0ABAFF",
            "#8CAAB6",
            "#8CAAB6",
            "#0ABAFF",
            "#0ABAFF",
            "#8CAAB6",
            "#8CAAB6",
            "#87AE87",
        ]

        self.annual_energy_expenses = self.ax.stackplot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "kerosene_cost"].fillna(0),
            self.df.loc[self.prospective_years, "kerosene_carbon_tax_cost"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_hefa_fog"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_fog"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_hefa_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_atj"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_atj"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_ft_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_others"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_cost_ft_msw"].fillna(0),
            self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_msw"].fillna(0),
            self.df.loc[self.prospective_years, "electrofuel_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "electrofuel_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "electrolysis_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "electrolysis_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "gas_ccs_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_ccs_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "gas_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "gas_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "coal_ccs_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_ccs_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "coal_h2_total_cost"].fillna(0),
            self.df.loc[self.prospective_years, "coal_h2_carbon_tax"].fillna(0),
            self.df.loc[self.prospective_years, "liquefaction_h2_total_cost"].fillna(0)
            + self.df.loc[self.prospective_years, "transport_h2_total_cost"].fillna(0),
            colors=colors,
            lw=0.5,
            edgecolor="black",
        )

        self.ax.grid(axis="y")
        self.ax.set_title("Annual energy expenses per pathway")
        self.ax.set_ylabel("Energy expenses [M€]")

        primary_legend_entries = [
            "Fossil Kerosene",
            "_nolegend_",
            "Bio - HEFA FOG",
            "_nolegend_",
            "Bio - HEFA Others",
            "_nolegend_",
            "Bio - Alcohol to Jet",
            "_nolegend_",
            "Bio - FT Others",
            "_nolegend_",
            "Bio - FT MSW",
            "_nolegend_",
            "Electrofuel",
            "_nolegend_",
            "Electrolysis $H_2$ ",
            "_nolegend_",
            "Gas CCS $H_2$ ",
            "_nolegend_",
            "Gas $H_2$ ",
            "_nolegend_",
            "Coal CCS $H_2$ ",
            "_nolegend_",
            "Coal $H_2$ ",
            "_nolegend_",
            "$H_2$ liq. & transport",
        ]

        stacks = self.annual_energy_expenses

        hatches = [
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
            "||",
            "",
        ]
        for stack, hatch in zip(stacks, hatches):
            stack.set_hatch(hatch)

        self.ax.set_xlim(2020, 2050)

        warnings.filterwarnings("ignore")

        primary_legend = self.ax.legend(primary_legend_entries, loc="upper left", prop={"size": 7})
        self.ax.add_artist(primary_legend)

        warnings.resetwarnings()

        # Create hatch legend manually
        hatch_patch = mpatches.Patch(facecolor="white", hatch="||", edgecolor="black")
        self.ax.legend(
            handles=[hatch_patch], labels=["Carbon Tax"], loc="upper right", prop={"size": 7}
        )

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class ScenarioEnergyCarbonTaxPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        (self.line_energy_expenses,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "non_discounted_energy_expenses"],
            label="Scenario energy expenses",
            linestyle="-",
            color="#0c9e30",
        )

        (self.line_energy_expenses_carb_tax,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "non_discounted_energy_expenses"]
                + self.df.loc[self.prospective_years, "kerosene_carbon_tax_cost"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_fog"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_others"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_atj"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_others"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_msw"]
                + self.df.loc[self.prospective_years, "electrolysis_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "gas_ccs_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "gas_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "coal_ccs_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "coal_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "electrofuel_carbon_tax"]
            ),
            label="Scenario energy expenses incl. carbon tax",
            linestyle="--",
            color="#0c9e30",
        )
        (self.line_bau_energy_expenses,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "non_discounted_BAU_energy_expenses"],
            label="Business as usual energy expenses",
            linestyle="-",
            color="#2A3438",
        )
        (self.line_bau_energy_expenses_carbon_tax,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "non_discounted_BAU_energy_expenses"]
                + self.df.loc[self.prospective_years, "kerosene_carbon_tax_BAU"]
            ),
            label="Business as usual energy expenses incl. carbon tax",
            linestyle="--",
            color="#2A3438",
        )

        self.ax.grid(axis="y")
        self.ax.legend(loc="upper left")
        self.ax.set_title("Annual energy expenses per pathway")
        self.ax.set_ylabel("Energy expenses [M€]")
        self.ax = plt.gca()

        self.ax.set_xlim(2020, 2050)
        # #
        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, df_data):
        self.df = df_data["vector_outputs"]

        self.line_energy_expenses.set_ydata(
            self.df.loc[self.prospective_years, "non_discounted_energy_expenses"]
        )

        self.line_energy_expenses_carb_tax.set_ydata(
            (
                self.df.loc[self.prospective_years, "non_discounted_energy_expenses"]
                + self.df.loc[self.prospective_years, "kerosene_carbon_tax_cost"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_fog"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_hefa_others"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_atj"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_others"]
                + self.df.loc[self.prospective_years, "biofuel_carbon_tax_ft_msw"]
                + self.df.loc[self.prospective_years, "electrolysis_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "gas_ccs_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "gas_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "coal_ccs_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "coal_h2_carbon_tax"]
                + self.df.loc[self.prospective_years, "electrofuel_carbon_tax"]
            )
        )

        self.line_bau_energy_expenses.set_ydata(
            self.df.loc[self.prospective_years, "non_discounted_BAU_energy_expenses"]
        )

        self.line_bau_energy_expenses_carbon_tax.set_ydata(
            (
                self.df.loc[self.prospective_years, "non_discounted_BAU_energy_expenses"]
                + self.df.loc[self.prospective_years, "kerosene_carbon_tax_BAU"]
            )
        )

        self.ax.collections.clear()
        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class ScenarioEnergyUnitCostPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        (self.line_kerosene_price,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "kerosene_market_price"] / 35.3,
            color="#2A3438",
            linestyle="-",
            label="Fossil Kerosene",
            linewidth=2,
        )

        (self.line_biofuel_hefa_fog_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "biofuel_hefa_fog_mfsp"] / 35.3,
            color="#097223",
            linestyle="-",
            label="Bio - HEFA FOG",
            linewidth=2,
        )

        (self.line_biofuel_hefa_others_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "biofuel_hefa_others_mfsp"] / 35.3,
            color="#097223",
            linestyle=":",
            label="Bio - HEFA Others",
            linewidth=2,
        )

        (self.line_biofuel_atj_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "biofuel_atj_mfsp"] / 35.3,
            color="#097223",
            linestyle="--",
            label="Bio - Alcohol to Jet",
            linewidth=2,
        )
        (self.line_biofuel_ft_others_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "biofuel_ft_others_mfsp"] / 35.3,
            color="#097223",
            linestyle="-.",
            label="Bio - FT Others",
            linewidth=2,
        )

        (self.line_biofuel_ft_msw_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "biofuel_ft_msw_mfsp"] / 35.3,
            color="#097223",
            linestyle=(0, (5, 10)),
            label="Bio - FT MSW",
            linewidth=2,
        )

        (self.line_electrofuel_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "electrofuel_avg_cost_per_l"] / 35.3,
            color="#828782",
            linestyle="-",
            label="Electrofuel",
            linewidth=2,
        )

        (self.line_hydrogen_electrolysis_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_electrolysis"] / 119.93,
            color="#0075A3",
            linestyle="-",
            label="Hydrogen - Electrolysis",
            linewidth=2,
        )

        (self.line_hydrogen_gas_ccs_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas_ccs"] / 119.93,
            color="#0075A3",
            linestyle=":",
            label="Hydrogen - Gas CSS",
            linewidth=2,
        )

        (self.line_hydrogen_gas_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas"] / 119.93,
            color="#0075A3",
            linestyle="--",
            label="Hydrogen - Gas",
            linewidth=2,
        )

        (self.line_hydrogen_coal_ccs_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal_ccs"] / 119.93,
            color="#0075A3",
            linestyle="-.",
            label="Hydrogen - Coal CCS",
            linewidth=2,
        )

        (self.line_hydrogen_coal_mfsp,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal"] / 119.93,
            color="#0075A3",
            linestyle=(0, (5, 10)),
            label="Hydrogen Coal",
            linewidth=2,
        )

        self.ax.grid(axis="y")
        self.ax.set_title("MFSP per pathway (kerosene: market price)")
        self.ax.set_ylabel("MFSP [€/MJ]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2020, 2050)
        # #
        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, df_data):
        self.df = df_data["vector_outputs"]

        self.line_kerosene_price.set_ydata(
            (self.df.loc[self.prospective_years, "kerosene_market_price"]) / 35.3,
        )
        self.line_biofuel_hefa_fog_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "biofuel_hefa_fog_mfsp"]) / 35.3,
        )
        self.line_biofuel_hefa_others_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "biofuel_hefa_others_mfsp"]) / 35.3,
        )
        self.line_biofuel_atj_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "biofuel_atj_mfsp"]) / 35.3,
        )
        self.line_biofuel_ft_others_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "biofuel_ft_others_mfsp"]) / 35.3,
        )
        self.line_biofuel_ft_msw_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "biofuel_ft_msw_mfsp"]) / 35.3,
        )
        self.line_electrofuel_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "electrofuel_avg_cost_per_l"]) / 35.3,
        )
        self.line_hydrogen_electrolysis_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_electrolysis"]) / 119.93,
        )
        self.line_hydrogen_gas_ccs_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas_ccs"]) / 119.93,
        )
        self.line_hydrogen_gas_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas"]) / 119.93,
        )
        self.line_hydrogen_coal_ccs_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal_ccs"]) / 119.93,
        )
        self.line_hydrogen_coal_mfsp.set_ydata(
            (self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal"]) / 119.93,
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class ScenarioEnergyUnitCostWithCarbonTaxPlot:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        ## With carbon tax

        (self.line_kerosene_price,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "kerosene_market_price"]
                + self.df.loc[self.prospective_years, "kerosene_price_supplement_carbon_tax"]
            )
            / 35.3,
            color="#2A3438",
            linestyle="-",
            label="Fossil Kerosene",
            linewidth=2,
        )

        (self.line_biofuel_hefa_fog_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "biofuel_hefa_fog_mfsp"]
                + self.df.loc[self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_hefa_fog"]
            )
            / 35.3,
            color="#097223",
            linestyle="-",
            label="Bio - HEFA FOG",
            linewidth=2,
        )

        (self.line_biofuel_hefa_others_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "biofuel_hefa_others_mfsp"]
                + self.df.loc[
                    self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_hefa_others"
                ]
            )
            / 35.3,
            color="#097223",
            linestyle=":",
            label="Bio - HEFA Others",
            linewidth=2,
        )

        (self.line_biofuel_atj_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "biofuel_atj_mfsp"]
                + self.df.loc[self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_atj"]
            )
            / 35.3,
            color="#097223",
            linestyle="--",
            label="Bio - Alcohol to Jet",
            linewidth=2,
        )
        (self.line_biofuel_ft_others_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "biofuel_ft_others_mfsp"]
                + self.df.loc[
                    self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_ft_others"
                ]
            )
            / 35.3,
            color="#097223",
            linestyle="-.",
            label="Bio - FT Others",
            linewidth=2,
        )

        (self.line_biofuel_ft_msw_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "biofuel_ft_msw_mfsp"]
                + self.df.loc[self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_ft_msw"]
            )
            / 35.3,
            color="#097223",
            linestyle=(0, (5, 10)),
            label="Bio - FT MSW",
            linewidth=2,
        )

        (self.line_electrofuel_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "electrofuel_avg_cost_per_l"]
                + self.df.loc[self.prospective_years, "electrofuel_mfsp_carbon_tax_supplement"]
            )
            / 35.3,
            color="#828782",
            linestyle="-",
            label="Electrofuel",
            linewidth=2,
        )

        (self.line_hydrogen_electrolysis_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_electrolysis"]
                + self.df.loc[self.prospective_years, "electrolysis_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
            color="#0075A3",
            linestyle="-",
            label="Hydrogen - Electrolysis",
            linewidth=2,
        )

        (self.line_hydrogen_gas_ccs_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas_ccs"]
                + self.df.loc[self.prospective_years, "gas_ccs_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
            color="#0075A3",
            linestyle=":",
            label="Hydrogen - Gas CSS",
            linewidth=2,
        )

        (self.line_hydrogen_gas_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas"]
                + +self.df.loc[self.prospective_years, "gas_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
            color="#0075A3",
            linestyle="--",
            label="Hydrogen - Gas",
            linewidth=2,
        )

        (self.line_hydrogen_coal_ccs_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal_ccs"]
                + self.df.loc[self.prospective_years, "coal_ccs_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
            color="#0075A3",
            linestyle="-.",
            label="Hydrogen - Coal CCS",
            linewidth=2,
        )

        (self.line_hydrogen_coal_mfsp,) = self.ax.plot(
            self.prospective_years,
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal"]
                + self.df.loc[self.prospective_years, "coal_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
            color="#0075A3",
            linestyle=(0, (5, 10)),
            label="Hydrogen Coal",
            linewidth=2,
        )

        self.ax.grid(axis="y")
        self.ax.set_title("MFSP per pathway incl. carbon tax")
        self.ax.set_ylabel("MFSP [€/MJ]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2020, 2050)
        # #
        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, df_data):
        self.df = df_data["vector_outputs"]

        self.line_kerosene_price.set_ydata(
            (
                self.df.loc[self.prospective_years, "kerosene_market_price"]
                + self.df.loc[self.prospective_years, "kerosene_price_supplement_carbon_tax"]
            )
            / 35.3,
        )
        self.line_biofuel_hefa_fog_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "biofuel_hefa_fog_mfsp"]
                + self.df.loc[self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_hefa_fog"]
            )
            / 35.3,
        )
        self.line_biofuel_hefa_others_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "biofuel_hefa_others_mfsp"]
                + self.df.loc[
                    self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_hefa_others"
                ]
            )
            / 35.3,
        )
        self.line_biofuel_atj_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "biofuel_atj_mfsp"]
                + self.df.loc[self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_atj"]
            )
            / 35.3,
        )
        self.line_biofuel_ft_others_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "biofuel_ft_others_mfsp"]
                + self.df.loc[
                    self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_ft_others"
                ]
            )
            / 35.3,
        )
        self.line_biofuel_ft_msw_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "biofuel_ft_msw_mfsp"]
                + self.df.loc[self.prospective_years, "biofuel_mfsp_carbon_tax_supplement_ft_msw"]
            )
            / 35.3,
        )
        self.line_electrofuel_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "electrofuel_avg_cost_per_l"]
                + self.df.loc[self.prospective_years, "electrofuel_mfsp_carbon_tax_supplement"]
            )
            / 35.3,
        )
        self.line_hydrogen_electrolysis_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_electrolysis"]
                + self.df.loc[self.prospective_years, "electrolysis_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
        )
        self.line_hydrogen_gas_ccs_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas_ccs"]
                + self.df.loc[self.prospective_years, "gas_ccs_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
        )
        self.line_hydrogen_gas_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_gas"]
                + self.df.loc[self.prospective_years, "gas_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
        )
        self.line_hydrogen_coal_ccs_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal_ccs"]
                + self.df.loc[self.prospective_years, "coal_ccs_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
        )
        self.line_hydrogen_coal_mfsp.set_ydata(
            (
                self.df.loc[self.prospective_years, "h2_avg_cost_per_kg_coal"]
                + self.df.loc[self.prospective_years, "coal_h2_mfsp_carbon_tax_supplement"]
            )
            / 119.93,
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class DiscountEffect:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.float_inputs = data["float_inputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        (self.line_non_discounted,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "non_discounted_energy_expenses"],
            color="#2A3438",
            linestyle="-",
            label="Non-discounted expenses",
            linewidth=2,
        )

        (self.line_discounted,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "discounted_energy_expenses"],
            color="red",
            linestyle="-",
            label="Discounted expenses at r={}%".format(self.float_inputs["social_discount_rate"]),
            linewidth=2,
        )

        self.ax.grid(axis="y")
        self.ax.set_title("Total (energy) expenses (M€)")
        self.ax.set_ylabel("M€ / year")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(2020, 2050)
        # #
        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, df_data):
        self.df = df_data["vector_outputs"]

        self.line_non_discounted.set_ydata(
            self.df.loc[self.prospective_years, "non_discounted_energy_expenses"]
        )

        self.line_discounted.set_ydata(
            self.df.loc[self.prospective_years, "discounted_energy_expenses"]
        )

        self.ax.collections.clear()

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class DropInMACC:
    # TODO do the same for LH2?
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        # Select year at which the MACC is plotted
        year = 2050

        # create a dataframe for the various pathways
        # (usage: sorting the values by increasing carbon abatement cost)

        pathways = [
            "Bio - HEFA FOG",
            "Bio - HEFA Others",
            "Bio - Alcohol to Jet",
            "Bio - FT MSW",
            "Bio - FT Others",
            "Electrofuel",
        ]

        # Abatement potential in MtCO2e
        abatement_potential = [
            elt / 1000000
            for elt in [
                self.df.abatement_potential_hefa_fog[year],
                self.df.abatement_potential_hefa_others[year],
                self.df.abatement_potential_atj[year],
                self.df.abatement_potential_ft_msw[year],
                self.df.abatement_potential_ft_others[year],
                self.df.abatement_potential_electrofuel[year],
            ]
        ]

        # Energy available in EJ
        energy_avail = [
            elt / 1000000000000
            for elt in [
                self.df.energy_avail_hefa_fog[year],
                self.df.energy_avail_hefa_others[year],
                self.df.energy_avail_atj[year],
                self.df.energy_avail_ft_msw[year],
                self.df.energy_avail_ft_others[year],
                self.df.energy_avail_electrofuel[year],
            ]
        ]

        # carbon abatement cost in (€/tCO2e)
        carbon_abatement_cost = [
            self.df.carbon_abatement_cost_hefa_fog[year],
            self.df.carbon_abatement_cost_hefa_others[year],
            self.df.carbon_abatement_cost_atj[year],
            self.df.carbon_abatement_cost_ft_msw[year],
            self.df.carbon_abatement_cost_ft_others[year],
            self.df.carbon_abatement_cost_electrofuel[year],
        ]

        colors = ["#ee9b00", "#ffbf47", "#bb3e03", "#097223", "#0c9e30", "#828782"]

        macc_df = pd.DataFrame(
            data=[abatement_potential, energy_avail, carbon_abatement_cost, colors],
            columns=pathways,
            index=["abatement_potential", "energy_avail", "carbon_abatement_cost", "colors"],
        )

        macc_df = macc_df.transpose().sort_values(by="carbon_abatement_cost")

        macc_df = macc_df[macc_df["abatement_potential"] > 0]

        heights = macc_df["carbon_abatement_cost"].to_list()
        names = macc_df.index.to_list()
        heights.insert(0, 0)
        heights.append(heights[-1])

        # MAx potential MACC
        widths_potential = macc_df["abatement_potential"].to_list()
        widths_potential.insert(0, 0)
        widths_potential.append(widths_potential[-1])

        colors = macc_df["colors"].to_list()

        self.macc_curve = self.ax.step(
            np.cumsum(widths_potential) - widths_potential,
            heights,
            where="post",
            color="#335C67",
            label="Marginal abatement cost",
            linewidth=1.5,
        )

        # Fill under the step plot with different colors for each step
        for i in range(0, (len(widths_potential) - 2)):
            # Create a polygon for each step
            polygon = plt.Polygon(
                [
                    (np.cumsum(widths_potential)[i], 0),
                    (np.cumsum(widths_potential)[i], heights[i + 1]),
                    (np.cumsum(widths_potential)[i + 1], heights[i + 1]),
                    (np.cumsum(widths_potential)[i + 1], 0),
                ],
                closed=True,
                color=colors[i],
                alpha=0.5,
            )
            self.ax.add_patch(polygon)

        fuel = macc_df.energy_avail.to_list()
        fuel.insert(0, 0)
        widths_potential.pop()
        self.ax2 = self.ax.twinx()
        self.ax2.plot(
            np.cumsum(widths_potential),
            np.cumsum(fuel),
            color="#9E2A2B",
            linestyle=":",
            label="Energy potential",
            marker="x",
        )

        self.ax2.axhline(
            y=self.df.energy_consumption_dropin_fuel[year] / 1e12
            - self.df.energy_consumption_kerosene[year] / 1e12,
            color="black",
            linewidth=1,
            linestyle="-.",
        )
        self.ax2.text(
            0,
            1.02
            * (
                self.df.energy_consumption_dropin_fuel[year] / 1e12
                - self.df.energy_consumption_kerosene[year] / 1e12
            ),
            "Air transport sustainable drop-in fuels use, 2050",
        )

        self.ax2.axhline(
            y=self.df.energy_consumption_dropin_fuel[year] / 1e12,
            color="black",
            linewidth=1,
            linestyle="-",
        )
        self.ax2.text(
            0,
            1.02 * (self.df.energy_consumption_dropin_fuel[year] / 1e12),
            "Air transport total drop-in fuels use, 2050",
        )

        self.ax.grid(True, which="both", ls=":")
        self.ax.set_ylabel("Carbon Abatement Cost (€/t$\mathregular{CO_2}$)")
        self.ax2.set_ylabel("Energy potential under current allocation (EJ)")
        self.ax.set_xlabel("$\mathregular{CO_2}$ abatted (Mt)")

        for i in range(len(widths_potential) - 1):
            self.ax.text(np.cumsum(widths_potential)[i] + 10, heights[i + 1] - 50, names[i])

        self.ax.legend(
            fancybox=True,
            shadow=True,
            loc="lower left",
            bbox_to_anchor=[0.00, -0.2],
            prop={"size": 8},
        )
        self.ax2.legend(
            fancybox=True,
            shadow=True,
            loc="lower right",
            bbox_to_anchor=[1, -0.2],
            prop={"size": 8},
        )

        self.ax.set_title("Marginal abatement cost curve for drop-in fuels")

        # #
        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, df_data):
        self.df = df_data["vector_outputs"]
        self.float_outputs = df_data["float_outputs"]
        self.years = df_data["years"]["full_years"]
        self.historic_years = df_data["years"]["historic_years"]
        self.prospective_years = df_data["years"]["prospective_years"]

        self.ax.cla()
        self.ax2.cla()

        # Select year at which the MACC is plotted
        year = 2050

        # create a dataframe for the various pathways
        # (usage: sorting the values by increasing carbon abatement cost)

        pathways = [
            "Bio - HEFA FOG",
            "Bio - HEFA Others",
            "Bio - Alcohol to Jet",
            "Bio - FT MSW",
            "Bio - FT Others",
            "Electrofuel",
        ]

        # Abatement potential in MtCO2e
        abatement_potential = [
            elt / 1000000
            for elt in [
                self.df.abatement_potential_hefa_fog[year],
                self.df.abatement_potential_hefa_others[year],
                self.df.abatement_potential_atj[year],
                self.df.abatement_potential_ft_msw[year],
                self.df.abatement_potential_ft_others[year],
                self.df.abatement_potential_electrofuel[year],
            ]
        ]

        # Energy available in EJ
        energy_avail = [
            elt / 1000000000000
            for elt in [
                self.df.energy_avail_hefa_fog[year],
                self.df.energy_avail_hefa_others[year],
                self.df.energy_avail_atj[year],
                self.df.energy_avail_ft_msw[year],
                self.df.energy_avail_ft_others[year],
                self.df.energy_avail_electrofuel[year],
            ]
        ]

        # carbon abatement cost in (€/tCO2e)
        carbon_abatement_cost = [
            self.df.carbon_abatement_cost_hefa_fog[year],
            self.df.carbon_abatement_cost_hefa_others[year],
            self.df.carbon_abatement_cost_atj[year],
            self.df.carbon_abatement_cost_ft_msw[year],
            self.df.carbon_abatement_cost_ft_others[year],
            self.df.carbon_abatement_cost_electrofuel[year],
        ]

        colors = ["#ee9b00", "#ffbf47", "#bb3e03", "#097223", "#0c9e30", "#828782"]

        macc_df = pd.DataFrame(
            data=[abatement_potential, energy_avail, carbon_abatement_cost, colors],
            columns=pathways,
            index=["abatement_potential", "energy_avail", "carbon_abatement_cost", "colors"],
        )

        macc_df = macc_df.transpose().sort_values(by="carbon_abatement_cost")

        macc_df = macc_df[macc_df["abatement_potential"] > 0]

        heights = macc_df["carbon_abatement_cost"].to_list()
        names = macc_df.index.to_list()
        heights.insert(0, 0)
        heights.append(heights[-1])

        # MAx potential MACC
        widths_potential = macc_df["abatement_potential"].to_list()
        widths_potential.insert(0, 0)
        widths_potential.append(widths_potential[-1])

        colors = macc_df["colors"].to_list()

        self.macc_curve = self.ax.step(
            np.cumsum(widths_potential) - widths_potential,
            heights,
            where="post",
            color="#335C67",
            label="Marginal abatement cost",
            linewidth=1.5,
        )

        # Fill under the step plot with different colors for each step
        for i in range(0, (len(widths_potential) - 2)):
            # Create a polygon for each step
            polygon = plt.Polygon(
                [
                    (np.cumsum(widths_potential)[i], 0),
                    (np.cumsum(widths_potential)[i], heights[i + 1]),
                    (np.cumsum(widths_potential)[i + 1], heights[i + 1]),
                    (np.cumsum(widths_potential)[i + 1], 0),
                ],
                closed=True,
                color=colors[i],
                alpha=0.5,
            )
            self.ax.add_patch(polygon)

        fuel = macc_df.energy_avail.to_list()
        fuel.insert(0, 0)
        widths_potential.pop()

        self.ax2.plot(
            np.cumsum(widths_potential),
            np.cumsum(fuel),
            color="#9E2A2B",
            linestyle=":",
            label="Energy potential",
            marker="x",
        )

        self.ax2.axhline(
            y=self.df.energy_consumption_dropin_fuel[year] / 1e12
            - self.df.energy_consumption_kerosene[year] / 1e12,
            color="black",
            linewidth=1,
            linestyle="-.",
        )
        self.ax2.text(
            0,
            1.02
            * (
                self.df.energy_consumption_dropin_fuel[year] / 1e12
                - self.df.energy_consumption_kerosene[year] / 1e12
            ),
            "Air transport sustainable drop-in fuels use, 2050",
        )

        self.ax2.axhline(
            y=self.df.energy_consumption_dropin_fuel[year] / 1e12,
            color="black",
            linewidth=1,
            linestyle="-",
        )
        self.ax2.text(
            0,
            1.02 * (self.df.energy_consumption_dropin_fuel[year] / 1e12),
            "Air transport total drop-in fuels use, 2050",
        )

        self.ax.grid(True, which="both", ls=":")
        self.ax.set_ylabel("Carbon Abatement Cost (€/t$\mathregular{CO_2}$)")
        self.ax2.set_ylabel("Energy potential under current allocation (EJ)")
        self.ax.set_xlabel("$\mathregular{CO_2}$ abatted (Mt)")

        for i in range(len(widths_potential) - 1):
            self.ax.text(np.cumsum(widths_potential)[i] + 10, heights[i + 1] - 50, names[i])

        self.fig.tight_layout()
        self.fig.canvas.draw()


class DOCEvolutionBreakdown:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):
        self.ax.plot(
            self.prospective_years,
            np.zeros(len(self.prospective_years)),
            color="royalblue",
            linestyle="-",
            linewidth=1,
        )

        (self.line_total,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_mean"],
            color="blue",
            linestyle="-",
            label="Total DOC",
            linewidth=2,
        )

        self.ax.fill_between(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"],
            np.zeros(len(self.prospective_years)),
            color="royalblue",
            label="Non-energy",
        )

        self.ax.fill_between(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_energy_per_ask_mean"],
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"],
            color="cornflowerblue",
            label="Energy",
        )

        self.ax.fill_between(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_carbon_tax_per_ask_mean"],
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_energy_per_ask_mean"],
            color="lightsteelblue",
            label="Carbon tax",
        )

        self.ax.grid()
        self.ax.set_title("Direct Operating Costs breakdown")
        self.ax.set_xlabel("Year")
        self.ax.set_ylabel("Direct Operating Costs [€/ASK]")
        self.ax = plt.gca()
        self.ax.legend()
        self.ax.set_xlim(self.prospective_years[0], self.prospective_years[-1])
        # self.ax.set_ylim(0,)

        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.ax.collections.clear()

        self.line_total.set_ydata(self.df.loc[self.prospective_years, "doc_total_per_ask_mean"])

        self.ax.fill_between(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"],
            np.zeros(len(self.prospective_years)),
            color="royalblue",
            label="Non-energy",
        )

        self.ax.fill_between(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_energy_per_ask_mean"],
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"],
            color="cornflowerblue",
            label="Energy",
        )

        self.ax.fill_between(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_carbon_tax_per_ask_mean"],
            self.df.loc[self.prospective_years, "doc_non_energy_per_ask_mean"]
            + self.df.loc[self.prospective_years, "doc_energy_per_ask_mean"],
            color="lightsteelblue",
            label="Carbon tax",
        )
        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()


class DOCEvolutionCategory:
    def __init__(self, data):
        self.df = data["vector_outputs"]
        self.float_outputs = data["float_outputs"]
        self.float_inputs = data["float_inputs"]
        self.years = data["years"]["full_years"]
        self.historic_years = data["years"]["historic_years"]
        self.prospective_years = data["years"]["prospective_years"]

        self.fig, self.ax = plt.subplots(
            figsize=(plot_3_x, plot_3_y),
        )
        self.create_plot()

    def create_plot(self):

        (self.line_srdi,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_short_range_dropin_fuel"],
            linestyle="-",
            label="Short range-D.in",
            linewidth=2,
        )

        (self.line_mrdi,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_medium_range_dropin_fuel"],
            linestyle="-",
            label="Medium range-D.in",
            linewidth=2,
        )

        (self.line_lrdi,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_long_range_dropin_fuel"],
            linestyle="-",
            label="Long range-D.in",
            linewidth=2,
        )

        (self.line_srh,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_short_range_hydrogen"],
            linestyle="-",
            label="Short range-H2",
            linewidth=2,
        )

        (self.line_mrh,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_medium_range_hydrogen"],
            linestyle="-",
            label="Medium range-H2",
            linewidth=2,
        )

        (self.line_lrh,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_long_range_hydrogen"],
            linestyle="-",
            label="Long range-H2",
            linewidth=2,
        )

        (self.line_tot,) = self.ax.plot(
            self.prospective_years,
            self.df.loc[self.prospective_years, "doc_total_per_ask_mean"],
            color="red",
            linestyle="-",
            label="Fleet average",
            linewidth=2,
        )

        self.ax.grid(axis="y")
        self.ax.set_title("Average direct operating cost by aircraft category")
        self.ax.set_ylabel("€ / ASK")
        self.ax = plt.gca()
        self.ax.legend(title="Direct Operating Cost")
        self.ax.set_xlim(2020, 2050)
        # #
        self.fig.canvas.header_visible = False
        self.fig.canvas.toolbar_position = "bottom"
        # self.fig.canvas.layout.width = "auto"
        # self.fig.canvas.layout.height = "auto"
        self.fig.tight_layout()

    def update(self, df_data):

        self.ax.collections.clear()

        self.df = df_data["vector_outputs"]

        self.line_lrh.set_ydata(
            self.df.loc[self.prospective_years, "doc_total_per_ask_long_range_hydrogen"]
        )

        self.line_lrdi.set_ydata(
            self.df.loc[self.prospective_years, "doc_total_per_ask_long_range_dropin_fuel"]
        )

        self.line_mrh.set_ydata(
            self.df.loc[self.prospective_years, "doc_total_per_ask_medium_range_hydrogen"]
        )

        self.line_mrdi.set_ydata(
            self.df.loc[self.prospective_years, "doc_total_per_ask_medium_range_dropin_fuel"]
        )

        self.line_srh.set_ydata(
            self.df.loc[self.prospective_years, "doc_total_per_ask_short_range_hydrogen"]
        )

        self.line_srdi.set_ydata(
            self.df.loc[self.prospective_years, "doc_total_per_ask_short_range_dropin_fuel"]
        )

        self.line_tot.set_ydata(self.df.loc[self.prospective_years, "doc_total_per_ask_mean"])

        self.ax.relim()
        self.ax.autoscale_view()
        self.fig.canvas.draw()
